# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2020-03-22 21:32
from __future__ import unicode_literals

from django.db import migrations, models
import is_core.contrib.background_export.models


class Migration(migrations.Migration):

    dependencies = [
        ('background_export', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='exportedfile',
            name='slug',
            field=models.SlugField(default=is_core.contrib.background_export.models.generate_slug, max_length=32,
                                   primary_key=True, serialize=False, verbose_name='slug'),
        ),
    ]
